// Code for Hello World on an oscilloscope in X-Y mode
//
// By howardlong (eevblog id) free to use for any purpose, but please note any other third party licensing conditions
// in particular in the fonts.h file
//
// Designed for PIC24FV16KM202 on 5V Microstick board DM240013-2
//
// Pin 23 is the X axis to oscilloscope
// Pin 25 is the Y axis to oscilloscope
// Pins 8 or 27 can be used for ground leads, or the black probe point on the board

#include <xc.h>
#include <stdint.h>
#include <ctype.h>

#define _MAIN
#include "main.h"
#include "font.h"

#pragma config FNOSC = FRC
#pragma config POSCMOD = NONE
#pragma config OSCIOFNC = CLKO
#pragma config FCKSM = CSECMD
#pragma config FWDTEN = OFF
#pragma config MCLRE = ON
#pragma config ICS = PGx1

#define abs(x) ((x)<0?-(x):(x))

void SetPixel(int nx,int ny)
{
	DAC1DAT=nx;
	DAC2DAT=255-ny;
}	

// Bressenham's pinched directly from Wikipedia.
void Line(int x0, int y0, int x1, int y1) 
{
	int dx = abs(x1-x0);
	int sx = x0<x1 ? 1 : -1;
	int dy = abs(y1-y0);
	int sy = y0<y1 ? 1 : -1; 
	int err = (dx>dy ? dx : -dy)/2;
	int e2;
	
	for(;;)
	{
		SetPixel(x0,y0);
		if (x0==x1 && y0==y1) 
		{
			break;
		}
		e2 = err;
		if (e2 >-dx)
		{ 
			err -= dy; 
			x0 += sx; 
		}
		if (e2 < dy) 
		{ 
			err += dx; 
			y0 += sy; 
		}
	}
}

void DrawChar(int *pxPos,int *pyPos, char c)
{
	if (c>=32 && c<=126)
	{
		int x1,y1,x2,y2;
		int nIdx=c-32; // Index into font array
		int nNumVerts=_au8FontNumBytes[nIdx]/4;
		const U8 *pu8=_apau8Font[nIdx];
		int i;
		int xPos=*pxPos; // Local copies to speed things up a little
		int yPos=*pyPos;
		
		if (xPos+_au8FontRealWidth[nIdx]>255) // Check to see if we neet to move to the next line
		{
			xPos=0; // Wrap horizontally
			yPos+=_u8FontHeight; // Move to next line
			if (yPos+_u8FontHeight>255)
			{
				yPos=0; // Wrap vertically
			}	
			*pyPos=yPos;
		}	
		
		for (i=0;i<nNumVerts;i++)
		{
			x1=*pu8++ + xPos;	
			y1=*pu8++ + yPos;	
			x2=*pu8++ + xPos;	
			y2=*pu8++ + yPos;
			
			Line(x1,y1,x2,y2);
		}
		
		*pxPos=xPos+_au8FontWidth[nIdx]; // Move to next character position for next time
	}
	else
	{
		switch (c)
		{
			case '\r':
				*pxPos=0;
				break;
			case '\t':
				*pxPos=(*pxPos+32) & 0xE0; // tab position
				if (*pxPos<255)
				{
					break;
				}
				// Deliberately NO BREAK HERE!!!
			case '\n':
				*pyPos=*pyPos+_u8FontHeight;
				if (*pyPos+_u8FontHeight>255)
				{
					*pyPos=0; // Wrap vertically
				}
				break;
		}		
	}	
}	

int main(void)
{
	// First set FOSC to 32MHz (i.e. FCY = 16MHz)
	CLKDIVbits.RCDIV=0b000; // div by 1 FRC postscaler
	
	__builtin_write_OSCCONH(0b00000001); // NOSC=FRCPLL
	__builtin_write_OSCCONL(0x00000001); // OSWEN=1
	
	while(OSCCONbits.COSC!=0b001)
	{
		Nop();
	}	
	while(OSCCONbits.LOCK!=1)
	{
		Nop();
	}
	
	// All I/Os digital
	ANSA=0;
	ANSB=0;

	TRISBbits.TRISB6=0; // Debug output pin
	
	DAC1CONbits.DACFM=0; // Lower 8 bits of DACxDAT used
	DAC1CONbits.DACTRIG=0; // DAC output updated as soon as written to
	DAC1CONbits.DACREF=0b10; // AVDD is reference
	DAC1CONbits.DACEN=1; // Enable DAC
	DAC1CONbits.DACOE=1; // DAC output enable
	
	DAC2CONbits.DACFM=0; // Lower 8 bits of DACxDAT used
	DAC2CONbits.DACTRIG=0; // DAC output updated as soon as written to
	DAC2CONbits.DACREF=0b10; // AVDD is reference
	DAC2CONbits.DACEN=1; // Enable DAC
	DAC2CONbits.DACOE=1; // DAC output enable

	while (1)
	{
		static char sz[]="Hello, world!\r\nThis is a test to see if this works. The quick brown fox jumps over the lazy dog. 0123456789";
		char *psz=sz;
		char c;
		int xPos=0;
		int yPos=0;
		
		while ((c=*psz++)!='\0')
		{
			DrawChar(&xPos,&yPos,c);
		}
		LATBbits.LATB6=!LATBbits.LATB6; // Toggle this line to determine frame rate
	}
	return 0;
}
